/*
 * lsrt.c - rt
 * list real-time processes, their priorities, and their scheduling policy
 *
 * Robert Love <rml@tech9.net>		27 April 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, v2, as
 * published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2002 Robert Love
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <limits.h>
#include <string.h>
#include <linux/unistd.h>
#include <getopt.h>

#include "include/lsrt.h"

void show_usage(const char * cmd)
{
	printf("lsrt version " VERSION "\n");
	printf("usage: %s [options]\n", cmd);
	printf("list real-time processes and their attributes\n");
	printf("  -f, --ff         only show SCHED_FIFO tasks\n");
	printf("  -r, --rr         only show SCHED_RR tasks\n");
	printf("  -o, --other      show SCHED_OTHER (normal) tasks\n");
	printf("  -h, --help       display this help\n");
	printf("  -v, --version    output version information\n");
	printf("Report bugs and send patches to <rml@tech9.net>\n");
}

/*
 * returns the command line of a given PID, upto len characters, and
 * puts the given command line, or NULL, in cmdline. returns NULL on error.
 */
char * get_cmdline(pid_t pid, char * cmdline, size_t len)
{
	char buf[PATH_MAX];
	FILE *f;

	snprintf(buf, PATH_MAX, "/proc/%d/cmdline", pid);
	f = fopen(buf, "r");
	if (f == NULL)
		return NULL;

	return fgets(cmdline, len, f);
}

extern int opind;

int main(int argc, char * argv[])
{
	pid_t p;
	struct sched_param sp;
	int policy, flags = 0, i;
	char cmdline[24];

	struct option longopts[] = {
		{ "ff",		0, NULL, 'f' },
		{ "help",	0, NULL, 'h' },
		{ "other",	0, NULL, 'o' },
		{ "rr",		0, NULL, 'r' },
		{ "version",	0, NULL, 'v' },
		{ NULL,		0, NULL, 0 }
	};

	while ((i = getopt_long(argc, argv, "fhorv", longopts, NULL)) != -1) {
		switch(i) {
			case 'f':
				flags |= FLAG_FIFO;
				flags &= ~FLAG_OTHER;
				flags &= ~FLAG_RR;
				break;
			case 'o':
				flags |= FLAG_OTHER;
				flags &= ~FLAG_FIFO;
				flags &= ~FLAG_RR;
				break;
			case 'r':
				flags |= FLAG_RR;
				flags &= ~FLAG_OTHER;
				flags &= ~FLAG_FIFO;
				break;
			case 'v':
				printf("lsrt version " VERSION "\n");
				return 0;
			case ':':
			case '?':
			case 'h':
				show_usage(argv[0]);
				return 1;
		}
	}

	printf("%-8s %-12s %-10s %-32s\n", "PID", "CLASS", "PRIORITY", "CMD");

	for (p = 0; p < MAX_PID; p++) {
		policy = sched_getscheduler(p);
		if (policy == -1)
			continue;

		if (!(flags & FLAG_OTHER) && policy == SCHED_OTHER)
			continue;
		if (flags & FLAG_RR && policy != SCHED_RR)
			continue;
		if (flags & FLAG_FIFO && policy != SCHED_FIFO)
			continue;

		if (sched_getparam(p, &sp) == -1) {
			perror("sched_getparam");
			return 1;
		}

		printf("%-8d ", p);

		switch (policy) {
			case SCHED_OTHER:
				printf("%-12s ", "(default)");
				break;
			case SCHED_FIFO:
				printf("%-12s ", "FIFO");
				break;
			case SCHED_RR:
				printf("%-12s ", "RR");
				break;
			default:
				printf("%-12s ", "(unknown)");
				break;
		}

		if (sched_getparam(p, &sp) == -1) {
			perror("sched_getparam");
			return 1;
		}
		printf("%-10d ", sp.sched_priority);

		if (!get_cmdline(p, cmdline, 32))
			strcpy(cmdline, "(unknown)");

		printf("%-32s\n", cmdline);
	}

	return 0;
}
