/*
 * rt.c - rt
 * Command-line utility for manipulating a task's real-time attributes 
 *
 * Robert Love <rml@tech9.net>		27 April 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, v2, as
 * published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2002 Robert Love
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <unistd.h>
#include <linux/unistd.h>
#include <getopt.h>

#include "include/rt.h"

void show_usage(const char * cmd)
{
	printf("rt version " VERSION "\n");
	printf("usage: %s [options] [prio] [pid | -- cmd [args...]]\n", cmd);
	printf("manipulate real-time attributes of a process\n");
	printf("  -f, --fifo     set policy of pid to SCHED_FF\n");
	printf("  -p, --pid      operate on existing given pid\n");
	printf("  -m, --max      show min and max valid priorities\n");
	printf("  -o, --other    set policy to SCHED_OTHER\n");
	printf("  -r, --rr       set policy to SCHED_RR (default)\n");
	printf("  -h, --help     display this help\n");
	printf("  -v, --version  output version information\n\n");
	printf("Report bugs and send patches to <rml@tech9.net>\n");
}

void show_rt_info(const char * what, pid_t pid)
{
	struct sched_param sp;
	int policy;

	/* don't display "pid 0" as that is confusing */
	if (!pid)
		pid = getpid();

	policy = sched_getscheduler(pid);
	if (policy == -1) {
		perror("sched_getscheduler");
		fprintf(stderr, "failed to get pid %d's policy\n", pid);
		exit(1);
	}

	printf("pid %d's %s scheduling policy: ", pid, what);
	switch (policy) {
		case SCHED_OTHER:
			printf("SCHED_OTHER\n");
			break;
		case SCHED_FIFO:
			printf("SCHED_FIFO\n");
			break;
		case SCHED_RR:
			printf("SCHED_RR\n");
			break;
		default:
			printf("unknown\n");
	}

	if (sched_getparam(pid, &sp)) {
		perror("sched_getparam");
		fprintf(stderr, "failed to get pid %d's attributes\n", pid);
		exit(1);
	}

	printf("pid %d's %s scheduling priority: %d\n",
			pid, what, sp.sched_priority);
}

extern int optind;

int main(int argc, char * argv[])
{
	int i, policy = SCHED_RR, priority = 0;
	struct sched_param sp;
	pid_t pid = 0;

	struct option longopts[] = {
		{ "fifo",	0, NULL, 'f' },
		{ "pid",	0, NULL, 'p' },
		{ "help",	0, NULL, 'h' },
		{ "max",        0, NULL, 'm' },
		{ "other",	0, NULL, 'o' },
		{ "rr",		0, NULL, 'r' },
		{ "version",	0, NULL, 'v' },
		{ NULL,		0, NULL, 0 }
	};

	while((i = getopt_long(argc, argv, "fphmorv", longopts, NULL)) != -1) {
		switch(i) {
			case 'f':
				policy = SCHED_FIFO;
				break;
			case 'm':
				printf("SCHED_FIFO  min/max prio:  %-3d %-3d\n"
				       "SCHED_RR    min/max prio:  %-3d %-3d\n"
				       "SCHED_OTHER min/max prio:  %-3d %-3d\n",
					sched_get_priority_min(SCHED_FIFO),
					sched_get_priority_max(SCHED_FIFO),
					sched_get_priority_min(SCHED_RR),
					sched_get_priority_max(SCHED_RR),
					sched_get_priority_min(SCHED_OTHER),
					sched_get_priority_max(SCHED_OTHER));
				return 0;
			case 'o':
				policy = SCHED_OTHER;
				break;
			case 'p':
				pid = atoi(argv[argc - 1]);
				break;
			case 'r':
				policy = SCHED_RR;
				break;
			case 'v':
				printf("rt version " VERSION "\n");
				return 0;
			case ':':
			case '?':
			case 'h':
				show_usage(argv[0]);
				return 1;
		}
		
	}

	if ((pid && argc - optind < 1) || (!pid && argc - optind < 2)) {
		show_usage(argv[0]);
		return 1;
	}

	if (pid) {
		show_rt_info("current", pid);
		if (argc - optind == 1)
			return 0;
	}

	priority = atoi(argv[optind]);

	sp.sched_priority = priority;
	if (sched_setscheduler(pid, policy, &sp) == -1) {
		perror("sched_setscheduler");
		fprintf(stderr, "failed to set pid %d's policy\n", pid);
		return 1;
	}

	show_rt_info("new", pid);

	if (!pid) {
		argv += optind + 1;
		execvp(argv[0], argv);
		perror("execvp");
		fprintf(stderr, "failed to execute %s\n", argv[0]);
		return 1;
	}

	return 0;
}
